'use strict'

const {
  asBoolean
} = require('./utils')

module.exports = {
  command: 'mkdir <path>',

  describe: 'Make mfs directories',

  builder: {
    parents: {
      alias: 'p',
      type: 'boolean',
      default: false,
      coerce: asBoolean,
      describe: 'No error if existing, make parent directories as needed.'
    },
    'cid-version': {
      alias: ['cid-ver'],
      type: 'number',
      default: 0,
      describe: 'Cid version to use. (experimental).'
    },
    'hash-alg': {
      type: 'string',
      describe: 'Hash function to use. Will set Cid version to 1 if used. (experimental).'
    },
    flush: {
      alias: 'f',
      type: 'boolean',
      default: true,
      coerce: asBoolean,
      describe: 'Flush the changes to disk immediately'
    },
    'shard-split-threshold': {
      type: 'number',
      default: 1000,
      describe: 'If a directory has more links than this, it will be transformed into a hamt-sharded-directory'
    }
  },

  handler (argv) {
    let {
      path,
      ipfs,
      parents,
      cidVersion,
      hashAlg,
      flush,
      shardSplitThreshold
    } = argv

    argv.resolve(
      ipfs.files.mkdir(path, {
        parents,
        cidVersion,
        hashAlg,
        flush,
        shardSplitThreshold
      })
    )
  }
}
