<!-- Generated by documentation.js. Update this documentation by updating the source code. -->

## SecureTrie

[src/secure.js:10-15][1]

**Extends Trie**

You can create a secure Trie where the keys are automatically hashed using **SHA3** by using `require('merkle-patricia-tree/secure')`. It has the same methods and constructor as `Trie`.

## Trie

[src/baseTrie.js:25-781][2]

Use `require('merkel-patricia-tree')` for the base interface. In Ethereum applications stick with the Secure Trie Overlay `require('merkel-patricia-tree/secure')`. The API for the raw and the secure interface are about the same

### Parameters

-   `db` **[Object][3]?** An instance of [levelup][4] or a compatible API. If the db is `null` or left undefined, then the trie will be stored in memory via [memdown][5]
-   `root` **([Buffer][6] \| [String][7])?** A hex `String` or `Buffer` for the root of a previously stored trie

### Properties

-   `root` **[Buffer][6]** The current root of the `trie`
-   `isCheckpoint` **[Boolean][8]** determines if you are saving to a checkpoint or directly to the db
-   `EMPTY_TRIE_ROOT` **[Buffer][6]** the Root for an empty trie

### get

[src/baseTrie.js:65-77][9]

Gets a value given a `key`

#### Parameters

-   `key` **([Buffer][6] \| [String][7])** the key to search for
-   `cb` **[Function][10]** A callback `Function` which is given the arguments `err` - for errors that may have occured and `value` - the found value in a `Buffer` or if no value was found `null`

### put

[src/baseTrie.js:87-111][11]

Stores a given `value` at the given `key`

#### Parameters

-   `key` **([Buffer][6] \| [String][7])** 
-   `Value` **([Buffer][6] \| [String][7])** 
-   `cb` **[Function][10]** A callback `Function` which is given the argument `err` - for errors that may have occured

### del

[src/baseTrie.js:120-136][12]

deletes a value given a `key`

#### Parameters

-   `key` **([Buffer][6] \| [String][7])** 
-   `callback` **[Function][10]** the callback `Function`

### getRaw

[src/baseTrie.js:145-162][13]

Retrieves a raw value in the underlying db

#### Parameters

-   `key` **[Buffer][6]** 
-   `callback` **[Function][10]** A callback `Function`, which is given the arguments `err` - for errors that may have occured and `value` - the found value in a `Buffer` or if no value was found `null`.

### putRaw

[src/baseTrie.js:192-201][14]

Writes a value directly to the underlining db

#### Parameters

-   `key` **([Buffer][6] \| [String][7])** The key as a `Buffer` or `String`
-   `value` **[Buffer][6]** The value to be stored
-   `callback` **[Function][10]** A callback `Function`, which is given the argument `err` - for errors that may have occured

### delRaw

[src/baseTrie.js:210-218][15]

Removes a raw value in the underlying db

#### Parameters

-   `key` **([Buffer][6] \| [String][7])** 
-   `callback` **[Function][10]** A callback `Function`, which is given the argument `err` - for errors that may have occured

### findPath

[src/baseTrie.js:252-298][16]

Tries to find a path to the node for the given key
It returns a `stack` of nodes to the closet node

#### Parameters

-   `null-null` **([String][7] \| [Buffer][6])** key - the search key
-   `null-null` **[Function][10]** cb - the callback function. Its is given the following
    arguments-   err - any errors encontered
    -   node - the last node found
    -   keyRemainder - the remaining key nibbles not accounted for
    -   stack - an array of nodes that forms the path to node we are searching for

### createReadStream

[src/baseTrie.js:730-732][17]

The `data` event is given an `Object` hat has two properties; the `key` and the `value`. Both should be Buffers.

Returns **[stream.Readable][18]** Returns a [stream][19] of the contents of the `trie`

### batch

[src/baseTrie.js:756-766][20]

The given hash of operations (key additions or deletions) are executed on the DB

#### Parameters

-   `ops` **[Array][21]** 
-   `cb` **[Function][10]** 

#### Examples

```javascript
var ops = [
   { type: 'del', key: 'father' }
 , { type: 'put', key: 'name', value: 'Yuri Irsenovich Kim' }
 , { type: 'put', key: 'dob', value: '16 February 1941' }
 , { type: 'put', key: 'spouse', value: 'Kim Young-sook' }
 , { type: 'put', key: 'occupation', value: 'Clown' }
]
trie.batch(ops)
```

### checkRoot

[src/baseTrie.js:775-780][22]

Checks if a given root exists

#### Parameters

-   `root` **[Buffer][6]** 
-   `cb` **[Function][10]** 

## Merkle Proof

Static functions for creating/verifying a merkle proof.


### prove

[src/proof.js:12-29][23]

Returns a merkle proof for a given key

#### Parameters

-   `trie` **[Trie][24]** 
-   `key` **[String][7]** 
-   `cb` **[Function][10]** A callback `Function` (arguments {Error} `err`, {Array.<TrieNode>} `proof`)

### verifyProof

[src/proof.js:39-100][25]

Verifies a merkle proof for a given key

#### Parameters

-   `rootHash` **[Buffer][6]** 
-   `key` **[String][7]** 
-   `proof` **[Array][21]&lt;TrieNode>** 
-   `cb` **[Function][10]** A callback `Function` (arguments {Error} `err`, {String} `val`)

## Internal Util Functions

These are not exposed.


### addHexPrefix

[src/trieNode.js:164-179][26]

#### Parameters

-   `key`  
-   `terminator`  
-   `dataArr` **[Array][21]** 

Returns **[Buffer][6]** returns buffer of encoded data
hexPrefix
\*

### asyncFirstSeries

[src/util.js:63-79][27]

Take a collection of async fns, call the cb on the first to return a truthy value.
If all run without a truthy result, return undefined

#### Parameters

-   `array`  
-   `iterator`  
-   `cb`  

### callTogether

[src/util.js:37-57][28]

Take two or more functions and returns a function  that will execute all of
the given functions

### doKeysMatch

[src/util.js:28-31][29]

Compare two 'nibble array' keys

#### Parameters

-   `keyA`  
-   `keyB`  

[1]: https://github.com/alextsg/merkle-patricia-tree/blob/c2da4c34d8001120dfc08aa45e2a9894a7927aab/src/secure.js#L10-L15 "Source code on GitHub"

[2]: https://github.com/alextsg/merkle-patricia-tree/blob/c2da4c34d8001120dfc08aa45e2a9894a7927aab/src/baseTrie.js#L25-L781 "Source code on GitHub"

[3]: https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Object

[4]: https://github.com/rvagg/node-levelup/

[5]: https://github.com/rvagg/memdown

[6]: https://nodejs.org/api/buffer.html

[7]: https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String

[8]: https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Boolean

[9]: https://github.com/alextsg/merkle-patricia-tree/blob/c2da4c34d8001120dfc08aa45e2a9894a7927aab/src/baseTrie.js#L65-L77 "Source code on GitHub"

[10]: https://developer.mozilla.org/docs/Web/JavaScript/Reference/Statements/function

[11]: https://github.com/alextsg/merkle-patricia-tree/blob/c2da4c34d8001120dfc08aa45e2a9894a7927aab/src/baseTrie.js#L87-L111 "Source code on GitHub"

[12]: https://github.com/alextsg/merkle-patricia-tree/blob/c2da4c34d8001120dfc08aa45e2a9894a7927aab/src/baseTrie.js#L120-L136 "Source code on GitHub"

[13]: https://github.com/alextsg/merkle-patricia-tree/blob/c2da4c34d8001120dfc08aa45e2a9894a7927aab/src/baseTrie.js#L145-L162 "Source code on GitHub"

[14]: https://github.com/alextsg/merkle-patricia-tree/blob/c2da4c34d8001120dfc08aa45e2a9894a7927aab/src/baseTrie.js#L192-L201 "Source code on GitHub"

[15]: https://github.com/alextsg/merkle-patricia-tree/blob/c2da4c34d8001120dfc08aa45e2a9894a7927aab/src/baseTrie.js#L210-L218 "Source code on GitHub"

[16]: https://github.com/alextsg/merkle-patricia-tree/blob/c2da4c34d8001120dfc08aa45e2a9894a7927aab/src/baseTrie.js#L252-L298 "Source code on GitHub"

[17]: https://github.com/alextsg/merkle-patricia-tree/blob/c2da4c34d8001120dfc08aa45e2a9894a7927aab/src/baseTrie.js#L730-L732 "Source code on GitHub"

[18]: https://nodejs.org/api/stream.html#stream_class_stream_readable

[19]: https://nodejs.org/dist/latest-v5.x/docs/api/stream.html#stream_class_stream_readable

[20]: https://github.com/alextsg/merkle-patricia-tree/blob/c2da4c34d8001120dfc08aa45e2a9894a7927aab/src/baseTrie.js#L756-L766 "Source code on GitHub"

[21]: https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Array

[22]: https://github.com/alextsg/merkle-patricia-tree/blob/c2da4c34d8001120dfc08aa45e2a9894a7927aab/src/baseTrie.js#L775-L780 "Source code on GitHub"

[23]: https://github.com/alextsg/merkle-patricia-tree/blob/c2da4c34d8001120dfc08aa45e2a9894a7927aab/src/proof.js#L12-L29 "Source code on GitHub"

[24]: #trie

[25]: https://github.com/alextsg/merkle-patricia-tree/blob/c2da4c34d8001120dfc08aa45e2a9894a7927aab/src/proof.js#L39-L100 "Source code on GitHub"

[26]: https://github.com/alextsg/merkle-patricia-tree/blob/c2da4c34d8001120dfc08aa45e2a9894a7927aab/src/trieNode.js#L164-L179 "Source code on GitHub"

[27]: https://github.com/alextsg/merkle-patricia-tree/blob/c2da4c34d8001120dfc08aa45e2a9894a7927aab/src/util.js#L63-L79 "Source code on GitHub"

[28]: https://github.com/alextsg/merkle-patricia-tree/blob/c2da4c34d8001120dfc08aa45e2a9894a7927aab/src/util.js#L37-L57 "Source code on GitHub"

[29]: https://github.com/alextsg/merkle-patricia-tree/blob/c2da4c34d8001120dfc08aa45e2a9894a7927aab/src/util.js#L28-L31 "Source code on GitHub"
